"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const net = require("net");
const cluster = require("cluster");
const connection_1 = require("./connection");
function rewriteConsole() {
    return __awaiter(this, void 0, void 0, function* () {
        const consoleErr = console.error;
        console.error = function () {
            if (cluster.isMaster) {
                consoleErr(`[mstr]`, ...arguments);
            }
            else {
                consoleErr(`[wkr${cluster.worker.id}]`, ...arguments);
            }
        };
    });
}
/**
 * serve starts a singleton language server instance that uses a
 * cluster of worker processes to achieve some semblance of
 * parallelism.
 */
function serve(options, newLangHandler) {
    return __awaiter(this, void 0, void 0, function* () {
        rewriteConsole();
        if (cluster.isMaster) {
            console.error(`Master node process spawning ${options.clusterSize} workers`);
            for (let i = 0; i < options.clusterSize; ++i) {
                const worker = cluster.fork().on('disconnect', () => {
                    console.error(`worker ${worker.process.pid} disconnect`);
                });
            }
            cluster.on('exit', (worker, code, signal) => {
                const reason = code === null ? signal : code;
                console.error(`worker ${worker.process.pid} exit (${reason})`);
            });
        }
        else {
            console.error('Listening for incoming LSP connections on', options.lspPort);
            var server = net.createServer((socket) => {
                const connection = connection_1.newConnection(socket, socket, options);
                connection_1.registerLanguageHandler(connection, options.strict, newLangHandler());
                connection.listen();
            });
            server.listen(options.lspPort);
        }
    });
}
exports.serve = serve;
//# sourceMappingURL=server.js.map